==> JVPC-0.93.zip: JVPC-0.93/vga/LSBWriter.java <==
// File: LSBWriter.java

package jvpc.vga;

import java.io.*;

/**
 * Writes Least Significant Byte First (LSB) numbers and Strings to an <CODE>
 * OutputStream</code>. A checksum is kept of all bytes written to the stream.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     13 XII 1998
 * @version   28 II 1999
 */
public class LSBWriter extends Writer {
  private OutputStream backup_os;
  private OutputStream os;
  private int position,partial,checksum;

  public LSBWriter(OutputStream os) {
    this.backup_os = os;
    this.os=new BufferedOutputStream(os);
  }

  public void flush() throws IOException {
    os.flush();
  }

  public void close() throws IOException {
    os.close();
  }

    public OutputStream getOutputStream() {
	return backup_os;
    }

  public void writeByte(byte b) throws IOException {
    os.write(b);
    position+=1;
    partial+=b;
    if (b<0) partial+=256;
  }

    public void writeByte(int i) throws IOException {
	writeByte((byte)i);
    }

  /**
   * Writes an unsigned byte.
   * @return The unsigned byte <EM>as an int</em>.
   */
  public void writeUnsignedByte(int i) throws IOException {
    if ((i<0) || (i>255)) 
      throw new IOException("The number "+i+" is not an unsigned byte");
    os.write((byte)i);
    position+=1;
    partial+=i;
  }

  /**
   * Writes a <CODE>short</code> in an LSB fashion.
   */
  public void writeShort(short sh) throws IOException {
    int one=(byte)(sh%256);
    int two=(byte)(sh>>8);
    os.write(one); // simulates unsigned byte behavior;
    os.write(two);
    position+=2;
    partial+=(sh%256)+(sh>>8);
    if (sh%256<0) partial+=256;
    if (sh>>8<0) partial+=256;
  }

  public void writeShort(int i) throws IOException {
    writeShort((short)i);
  }
  
  /**
   * Writes an unsigned <CODE>short</code> in an LSB fashion.
   * @return The unsigned short <EM>as an int</em>.
   */
  public void writeUnsignedShort(int sh) throws IOException {
    // build in check for ``overflow'' ...
    int one=(byte)(sh%256);
    int two=(byte)(sh>>8);
    os.write(one); // simulates unsigned byte behavior;
    os.write(two);
    position+=2;
    partial+=(sh%256)+(sh>>8);
    if (sh%256<0) partial+=256;
    if (sh>>8<0) partial+=256;
  }

  /**
   * Writes an <CODE>int</code> in an LSB fashion.
   */
  public void writeInt(int i) throws IOException {
    int one=(byte)(i%256);
    int two=(byte)(i>>8);
    int three=(byte)(i>>16);
    int four=(byte)(i>>24);
    os.write(one); // simulates unsigned byte behavior
    os.write(two);
    os.write(three);
    os.write(four);
    position+=4;
    partial+=(i%256)+(i>>8)%256+(i>>16)%256+(i>>24)%256;
    if (i%256<0) partial+=256;
    if ((i>>8)%256<0) partial+=256;
    if ((i>>16)%256<0) partial+=256;
    if ((i>>24)%256<0) partial+=256;
  }

  /**
   * Writes an unsigned <CODE>int</code> in an LSB fashion.
   * @return The unsigned int <EM>as a long</em>.
   */
  public void writeUnsignedInt(long i) throws IOException {
    int one=(byte)(i%256);
    int two=(byte)(i>>8);
    int three=(byte)(i>>16);
    int four=(byte)(i>>24);
    os.write(one); // simulates unsigned byte behavior
    os.write(two);
    os.write(three);
    os.write(four);
    position+=4;
    partial+=(i%256)+(i>>8)%256+(i>>16)%256+(i>>24)%256;
  }

  /**
   * Writes a <CODE>long</code> in an LSB fashion.
   * @exception @see java.io.OutputStream.write()
   */
  public void writeLong(long l) throws IOException {
    int one=(byte)(l%256);
    int two=(byte)(l>>8);
    int three=(byte)(l>>16);
    int four=(byte)(l>>24);
    int five=(byte)(l>>32);
    int six=(byte)(l>>40);
    int seven=(byte)(l>>48);
    int eight=(byte)(l>>56);
    os.write(one); // simulates unsigned byte behavior
    os.write(two);
    os.write(three);
    os.write(four);
    os.write(five);
    os.write(six);
    os.write(seven);
    os.write(eight);
    position+=8;
    partial+=one+two+three+four+five+six+seven+eight;
  }

    /**
     * Writes a String, converts bytes to chars.
     */
    public void writeString(String string) throws IOException {
	System.out.println("writing string of length "+string.length());
	writeByteArray(string.getBytes());
    }

  /**
   * Writes a String, converts bytes to chars.
   */
  public void writeString(String string,int size) throws IOException {
    writeString(new StringBuffer(string),size);
  }

  /**
   * Writes a StringBuffer, converts bytes to chars.
   */
  public void writeString(StringBuffer sb) throws IOException {
    writeString(sb,sb.length());
  }

  /**
   * Writes a StringBuffer, converts bytes to chars.
   */
  public void writeString(StringBuffer sb,int size) throws IOException {
    if (size>sb.length()) return;
    int i;
    int ch;
    for (i=0;i<size;i++) {
      // write ith byte, convert it to UNIcode char
      ch=sb.charAt(i); // plain ASCII -> conversion succesfull
      os.write(ch);
      partial+=((int)ch<0?(int)ch+256:(int)ch);
    }
    position+=size;
  }

    public void writeByteArray(byte[] ba) throws IOException {
	System.err.println("writing byte array of length "+ba.length);
	int size = ba.length;
	os.write(ba);
	position += size;
	for (int i=0; i<size; i++) {
	    partial+=ba[i];
	    if (ba[i]<0) partial+=256;
	}
    }

  public void write(char[] string, int size, int amount) throws IOException {
    throw new IOException("Not implemented");
  }

  /**
   * Skips a number of bytes.
   *
   * @param size Amount of skipped bytes.
   */
  public long skip(long size) throws IOException {
    for (int i=0;i<size;i++) os.write(0); // Implement this somehow...
    position += size;
    // can't add to partial, unknown bytes passing underneath
    // "implement this somehow" gives zero, anyway
    return 0;
  }

    public int getPosition() {
	return position;
    }

  public void resetChecksum() {
    partial=checksum=0;
  }

  public void resetPartial() {
    checksum+=partial;
    partial=0;
  }

  public int getAndResetPartial() {
    int result=partial;
    checksum+=partial;
    partial=0;
    return result;
  }

  public int getChecksum() {
    return partial+checksum;
  }
}

==> JVPC-0.94.zip: JVPC-0.94/vga/LSBWriter.java <==
// File: LSBWriter.java

package jvpc.vga;

import java.io.*;

/**
 * Writes Least Significant Byte First (LSB) numbers and Strings to an <CODE>
 * OutputStream</code>. A checksum is kept of all bytes written to the stream.
 *
 * @author    Kero van Gelder
 * @copyright Kero van Gelder (the GNU General Public License applies)
 * @since     13 XII 1998
 * @version   28 II 1999
 */
public class LSBWriter extends Writer {
  private OutputStream backup_os;
  private OutputStream os;
  private int position,partial,checksum;

  public LSBWriter(OutputStream os) {
    this.backup_os = os;
    this.os=new BufferedOutputStream(os);
  }

  public void flush() throws IOException {
    os.flush();
  }

  public void close() throws IOException {
    os.close();
  }

    public OutputStream getOutputStream() {
	return backup_os;
    }

  public void writeByte(byte b) throws IOException {
    os.write(b);
    position+=1;
    partial+=b;
    if (b<0) partial+=256;
  }

    public void writeByte(int i) throws IOException {
	writeByte((byte)i);
    }

  /**
   * Writes an unsigned byte.
   * @return The unsigned byte <EM>as an int</em>.
   */
  public void writeUnsignedByte(int i) throws IOException {
    if ((i<0) || (i>255)) 
      throw new IOException("The number "+i+" is not an unsigned byte");
    os.write((byte)i);
    position+=1;
    partial+=i;
  }

  /**
   * Writes a <CODE>short</code> in an LSB fashion.
   */
  public void writeShort(short sh) throws IOException {
    int one=(byte)(sh%256);
    int two=(byte)(sh>>8);
    os.write(one); // simulates unsigned byte behavior;
    os.write(two);
    position+=2;
    partial+=(sh%256)+(sh>>8);
    if (sh%256<0) partial+=256;
    if (sh>>8<0) partial+=256;
  }

  public void writeShort(int i) throws IOException {
    writeShort((short)i);
  }
  
  /**
   * Writes an unsigned <CODE>short</code> in an LSB fashion.
   * @return The unsigned short <EM>as an int</em>.
   */
  public void writeUnsignedShort(int sh) throws IOException {
    // build in check for ``overflow'' ...
    int one=(byte)(sh%256);
    int two=(byte)(sh>>8);
    os.write(one); // simulates unsigned byte behavior;
    os.write(two);
    position+=2;
    partial+=(sh%256)+(sh>>8);
    if (sh%256<0) partial+=256;
    if (sh>>8<0) partial+=256;
  }

  /**
   * Writes an <CODE>int</code> in an LSB fashion.
   */
  public void writeInt(int i) throws IOException {
    int one=(byte)(i%256);
    int two=(byte)(i>>8);
    int three=(byte)(i>>16);
    int four=(byte)(i>>24);
    os.write(one); // simulates unsigned byte behavior
    os.write(two);
    os.write(three);
    os.write(four);
    position+=4;
    partial+=(i%256)+(i>>8)%256+(i>>16)%256+(i>>24)%256;
    if (i%256<0) partial+=256;
    if ((i>>8)%256<0) partial+=256;
    if ((i>>16)%256<0) partial+=256;
    if ((i>>24)%256<0) partial+=256;
  }

  /**
   * Writes an unsigned <CODE>int</code> in an LSB fashion.
   * @return The unsigned int <EM>as a long</em>.
   */
  public void writeUnsignedInt(long i) throws IOException {
    int one=(byte)(i%256);
    int two=(byte)(i>>8);
    int three=(byte)(i>>16);
    int four=(byte)(i>>24);
    os.write(one); // simulates unsigned byte behavior
    os.write(two);
    os.write(three);
    os.write(four);
    position+=4;
    partial+=(i%256)+(i>>8)%256+(i>>16)%256+(i>>24)%256;
  }

  /**
   * Writes a <CODE>long</code> in an LSB fashion.
   * @exception @see java.io.OutputStream.write()
   */
  public void writeLong(long l) throws IOException {
    int one=(byte)(l%256);
    int two=(byte)(l>>8);
    int three=(byte)(l>>16);
    int four=(byte)(l>>24);
    int five=(byte)(l>>32);
    int six=(byte)(l>>40);
    int seven=(byte)(l>>48);
    int eight=(byte)(l>>56);
    os.write(one); // simulates unsigned byte behavior
    os.write(two);
    os.write(three);
    os.write(four);
    os.write(five);
    os.write(six);
    os.write(seven);
    os.write(eight);
    position+=8;
    partial+=one+two+three+four+five+six+seven+eight;
  }

    /**
     * Writes a String, converts bytes to chars.
     */
    public void writeString(String string) throws IOException {
	//System.out.println("writing string of length "+string.length());
	writeByteArray(string.getBytes());
    }

  /**
   * Writes a String, converts bytes to chars.
   */
  public void writeString(String string,int size) throws IOException {
    writeString(new StringBuffer(string),size);
  }

  /**
   * Writes a StringBuffer, converts bytes to chars.
   */
  public void writeString(StringBuffer sb) throws IOException {
    writeString(sb,sb.length());
  }

  /**
   * Writes a StringBuffer, converts bytes to chars.
   */
  public void writeString(StringBuffer sb,int size) throws IOException {
    if (size>sb.length()) return;
    int i;
    int ch;
    for (i=0;i<size;i++) {
      // write ith byte, convert it to UNIcode char
      ch=sb.charAt(i); // plain ASCII -> conversion succesfull
      os.write(ch);
      partial+=((int)ch<0?(int)ch+256:(int)ch);
    }
    position+=size;
  }

    public void writeByteArray(byte[] ba) throws IOException {
	//System.err.println("writing byte array of length "+ba.length);
	int size = ba.length;
	os.write(ba);
	position += size;
	for (int i=0; i<size; i++) {
	    partial+=ba[i];
	    if (ba[i]<0) partial+=256;
	}
    }

  public void write(char[] string, int size, int amount) throws IOException {
    throw new IOException("Not implemented");
  }

  /**
   * Skips a number of bytes.
   *
   * @param size Amount of skipped bytes.
   */
  public long skip(long size) throws IOException {
    for (int i=0;i<size;i++) os.write(0); // Implement this somehow...
    position += size;
    // can't add to partial, unknown bytes passing underneath
    // "implement this somehow" gives zero, anyway
    return 0;
  }

    public int getPosition() {
	return position;
    }

  public void resetChecksum() {
    partial=checksum=0;
  }

  public void resetPartial() {
    checksum+=partial;
    partial=0;
  }

  public int getAndResetPartial() {
    int result=partial;
    checksum+=partial;
    partial=0;
    return result;
  }

  public int getChecksum() {
    return partial+checksum;
  }
}
